#include "PHImage.h"
#include "PHGeometry.h"
#include "PHAffineTransform.h"

#define INTEGRAL_SIZE

#ifdef INTEGRAL_SIZE
#include <math.h>
#endif

@implementation NSImage (PhotoClip)

- (NSImage *) imageFromRect: (NSRect) rect
{
    NSImage * canvas = [[NSImage alloc] initWithSize: rect.size];
    [canvas lockFocus];

    [self compositeToPoint: NSZeroPoint
          fromRect: rect
          operation: NSCompositeCopy];

    [canvas unlockFocus];
    return [canvas autorelease];
}

- (NSImage *) imageTransformedBy: (NSAffineTransform *) transform
{
    NSSize new_size = [transform transformSize: [self size]];
    NSImage * canvas = [[NSImage alloc] initWithSize: new_size];
    [canvas lockFocus];

    [transform concat];

    // Get NSImageRep of image
    NSImageRep * rep = [self bestRepresentationForDevice: nil];

    [rep drawAtPoint: NSZeroPoint];
    
    [canvas unlockFocus];
    return [canvas autorelease];
}


// This method (transformation after clip) does not work

- (NSImage *) imageFromRect: (NSRect) rect
              transformedBy: (NSAffineTransform *) transform
{
    NSRect original_rect;
    original_rect.origin = NSZeroPoint;
    original_rect.size = [self size];

    if ( NSEqualRects(original_rect, NSIntersectionRect(original_rect, rect)) )
        return [self imageTransformedBy: transform]; // nothing to clip

    // First, clip the scene
    
    NSImage * fragment = [[NSImage alloc] initWithSize: rect.size];
    [fragment lockFocus];

    [self compositeToPoint: NSZeroPoint
          fromRect: rect
          operation: NSCompositeCopy];

    [fragment unlockFocus];

    // Then, transform.
    NSImage * result = [fragment imageTransformedBy: transform];
    RELEASE(fragment);
    
    return result;
}

- (NSImage *) imageFromRect: (NSRect) rect
                  rotatedBy: (float) degrees
{
    [self imageFromRect: rect scaledBy: 1.0 rotatedBy: degrees];
}

- (NSImage *) imageFromRect: (NSRect) rect
                   scaledBy: (float) scale
                  rotatedBy: (float) degrees
{
    NSAffineTransform * xform = [NSAffineTransform transformToQIofRect: rect
                                                   scale: scale
                                                   rotation: degrees];
    
    NSSize canvas_size = [xform transformSize: rect.size];

#ifdef INTEGRAL_SIZE
    canvas_size.width = ceilf(canvas_size.width);
    canvas_size.height = ceilf(canvas_size.height);
#endif // INTEGRAL_SIZE

    NSImage * canvas = [[NSImage alloc] initWithSize: canvas_size];
    [canvas lockFocus];

    [xform concat];
    
    // Get NSImageRep of image
    NSImageRep * rep = [self bestRepresentationForDevice: nil];

    [rep drawAtPoint: NSZeroPoint];
    
    [canvas unlockFocus];
    return AUTORELEASE(canvas);
}

- (NSAffineTransform *) transformFromRotation: (float) degrees
                                        scale: (float) scale
{
    return [NSAffineTransform transformToQIofSize: [self size]
                              scale: scale rotation: degrees];
}

@end
